
(defun modify-file-local-variable (variable value op)
  "Modify file-local VARIABLE in Local Variables depending on operation OP.

If OP is `add-or-replace' then delete all existing settings of
VARIABLE and add a new file-local VARIABLE with VALUE to the
Local Variables list.

If there is no Local Variables list in the current file buffer and OP
is not `delete' then this function adds the first line containing the
string `Local Variables:' and the last line containing the string `End:'.

If OP is `delete' then delete all existing settings of VARIABLE
from the Local Variables list ignoring the input argument VALUE."
  (catch 'exit
    (let ((startpos (point)) endpos replaced-p)
      (unless enable-local-variables
	(throw 'exit (message "File-local variables are disabled")))

      ;; Look for "Local variables:" line in last page.
      (goto-char (point-max))
      (search-backward "\n\^L" (max (- (point-max) 3000) (point-min))
		       'move)

      ;; Add "Local variables:" list if not found.
      (unless (let ((case-fold-search t))
		(search-forward "Local Variables:" nil t))

	;; Don't add "Local variables:" list for the deletion operation.
	(when (eq op 'delete)
	  (throw 'exit (progn (goto-char startpos)
			      (message "Local Variables not found"))))

	(goto-char (point-max))
	(let ((comment-style 'plain)
	      (comment-start (or comment-start ";;; ")))
	  (comment-region
	   (prog1 (point)
	     (insert "\nLocal Variables:\nEnd:\n"))
	   (point)))

	(unless (let ((case-fold-search t))
		  (and (search-backward "Local Variables:" nil t)
		       (search-forward "Local Variables:" nil t)))
	  (throw 'exit (progn (goto-char startpos)
			      (message "Can't add file-local variables")))))

      ;; suffix is what comes after "local variables:" in its line.
      ;; prefix is what comes before "local variables:" in its line.
      (let* ((suffix (buffer-substring (point) (line-end-position)))
	     (prefix (buffer-substring (line-beginning-position)
				       (match-beginning 0)))
	     (suffix-re (concat (regexp-quote suffix) "$"))
	     (prefix-re (concat "^" (regexp-quote prefix))))

	;; Find or add missing "End:".
	(forward-line 1)
	(setq startpos (point))
	(save-excursion
	  (unless (let ((case-fold-search t))
		    (re-search-forward
		     (concat prefix-re "[ \t]*End:[ \t]*" suffix-re)
		     nil t))
	    (save-excursion
	      (insert (format "%sEnd:%s\n" prefix suffix))))
	  (beginning-of-line)
	  (setq endpos (point-marker)))

	;; Find and delete all existing variable/value pairs.
	(when (member op '(add-or-replace delete))
	  (goto-char startpos)
	  (while (re-search-forward
		  (format "%s%S:.*%s" prefix-re variable suffix-re) endpos t)
	    (delete-region (match-beginning 0) (1+ (match-end 0)))
	    (setq replaced-p t)))

	;; Add a new variable/value pair.  Add `mode' to the start, add new
	;; variable to the end, and add a replaced variable to its last location.
	(when (eq op 'add-or-replace)
	  (cond
	   ((eq variable 'mode) (goto-char startpos))
	   ((null replaced-p) (goto-char endpos)))
	  (insert (format "%s%S: %S%s\n" prefix variable value suffix)))))))

(defun add-file-local-variable (variable value)
  "Add file-local VARIABLE with its VALUE to the Local Variables list.

This command deletes all existing settings of VARIABLE and adds a new
file-local VARIABLE with VALUE to the Local Variables list.

If there is no Local Variables list in the current file buffer
then this function adds the first line containing the string
`Local Variables:' and the last line containing the string `End:'."
  (interactive
   (let* ((default (variable-at-point))
	  (variable
	   (if (and (symbolp default) (boundp default))
	       (read-variable
		(format "Add file-local variable (default %s): " default)
		default)
	     (read-variable "Add file-local variable: ")))
	  (value
	   (read (read-string (format "Add %s with value: " variable)
			      nil 'set-variable-value-history
			      (format "%S" (symbol-value variable))))))
     (list variable value)))
  (modify-file-local-variable variable value 'add-or-replace))

(defun delete-file-local-variable (variable)
  "Delete all settings of file-local VARIABLE from the Local Variables list."
  (interactive
   (let* ((default (variable-at-point))
	  (variable
	   (if (and (symbolp default) (boundp default))
	       (read-variable
		(format "Delete file-local variable (default %s): " default)
		default)
	     (read-variable "Delete file-local variable: "))))
     (list variable)))
  (modify-file-local-variable variable nil 'delete))
